"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Disjunction = void 0;
var Disjunction = (function () {
    function Disjunction(matchables, negated) {
        if (negated === void 0) { negated = false; }
        this.matchables = matchables;
        this.negated = negated;
    }
    Disjunction.prototype.matchRight = function (stat, from, module) {
        var _this = this;
        var to = (function () {
            if (_this.matchables.length > 0) {
                for (var _i = 0, _a = _this.matchables.slice().reverse(); _i < _a.length; _i++) {
                    var matchable = _a[_i];
                    var singleTo = matchable.matchRight(stat, from, module);
                    if (singleTo >= 0) {
                        return singleTo;
                    }
                }
                return -1;
            }
            else {
                return -1;
            }
        })();
        if (this.negated) {
            return (to === -1) ? from : -1;
        }
        else {
            return to;
        }
    };
    Disjunction.prototype.matchLeft = function (stat, to, module) {
        var _this = this;
        var from = (function () {
            if (_this.matchables.length > 0) {
                for (var _i = 0, _a = _this.matchables.slice().reverse(); _i < _a.length; _i++) {
                    var matchable = _a[_i];
                    var singleFrom = matchable.matchLeft(stat, to, module);
                    if (singleFrom >= 0) {
                        return singleFrom;
                    }
                }
                return -1;
            }
            else {
                return -1;
            }
        })();
        if (this.negated) {
            return (from === -1) ? to : -1;
        }
        else {
            return from;
        }
    };
    Disjunction.prototype.isConcrete = function () {
        return this.matchables.length >= 2 || (this.matchables.length >= 1 && this.matchables[0].isConcrete());
    };
    Disjunction.prototype.findUnknownIdentifier = function (module) {
        for (var _i = 0, _a = this.matchables; _i < _a.length; _i++) {
            var matchable = _a[_i];
            var identifier = matchable.findUnknownIdentifier(module);
            if (identifier !== undefined) {
                return identifier;
            }
        }
        return undefined;
    };
    Disjunction.prototype.findCircularIdentifier = function (identifiers, module) {
        for (var _i = 0, _a = this.matchables; _i < _a.length; _i++) {
            var matchable = _a[_i];
            var identifier = matchable.findCircularIdentifier(identifiers, module);
            if (identifier !== undefined) {
                return identifier;
            }
        }
        return undefined;
    };
    Disjunction.prototype.toString = function () {
        var string = "";
        if (this.negated) {
            string += "!";
        }
        string += "(" + this.matchables.join(" | ") + ")";
        return string;
    };
    return Disjunction;
}());
exports.Disjunction = Disjunction;
