"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Module = void 0;
var _1 = require(".");
var Module = (function () {
    function Module(name, sentences) {
        this.definitions = [];
        this.rules = [];
        this.environments = [];
        this.name = name;
        var _loop_1 = function (sentence) {
            if (sentence instanceof _1.ModuleChain) {
                if (this_1.definitions.length > 0 || this_1.rules.length > 0) {
                    throw new _1.AkrantiainError(1005, -1, "Module has both sentences and a module chain: '" + name + "'");
                }
            }
            else {
                if (this_1.moduleChain !== undefined) {
                    throw new _1.AkrantiainError(1005, -1, "Module has both sentences and a module chain: '" + name + "'");
                }
            }
            if (sentence instanceof _1.Definition) {
                var duplicatedIndex = this_1.definitions.findIndex(function (definition) {
                    var castSentence = sentence;
                    return definition.identifier.equals(castSentence.identifier);
                });
                if (duplicatedIndex < 0) {
                    this_1.definitions.push(sentence);
                }
                else {
                    throw new _1.AkrantiainError(1103, 210, "Duplicate definition of identifier: '" + sentence.identifier + "'");
                }
            }
            else if (sentence instanceof _1.Rule) {
                this_1.rules.push(sentence);
            }
            else if (sentence instanceof _1.Environment) {
                this_1.environments.push(sentence);
            }
            else if (sentence instanceof _1.ModuleChain) {
                if (this_1.moduleChain === undefined) {
                    this_1.moduleChain = sentence;
                }
                else {
                    throw new _1.AkrantiainError(1006, -1, "Module has multiple module chains: '" + name + "'");
                }
            }
            else {
                throw new Error("This cannot happen");
            }
        };
        var this_1 = this;
        for (var _i = 0, sentences_1 = sentences; _i < sentences_1.length; _i++) {
            var sentence = sentences_1[_i];
            _loop_1(sentence);
        }
        this.checkUnknownIdentifier();
        this.checkCircularIdentifier();
    }
    Module.prototype.convert = function (input, akrantiain) {
        var currentOutput = input;
        currentOutput = this.convertByRule(currentOutput, akrantiain);
        currentOutput = this.convertByModuleChain(currentOutput, akrantiain);
        return currentOutput;
    };
    Module.prototype.convertByRule = function (input, akrantiain) {
        if (this.rules.length > 0) {
            var currentStat = _1.Stat.create(input, this);
            for (var _i = 0, _a = this.rules; _i < _a.length; _i++) {
                var rule = _a[_i];
                currentStat = rule.apply(currentStat, this);
            }
            var invalidElements = currentStat.getInvalidElements(this);
            if (invalidElements.length <= 0 || this.hasEnvironment("FALL_THROUGH")) {
                return currentStat.createOutput(this);
            }
            else {
                throw new _1.AkrantiainError(2000, 210, "No rules that can handle some characters", invalidElements);
            }
        }
        else {
            return input;
        }
    };
    Module.prototype.convertByModuleChain = function (input, akrantiain) {
        var currentOutput = input;
        if (this.moduleChain !== undefined) {
            for (var _i = 0, _a = this.moduleChain.names; _i < _a.length; _i++) {
                var name_1 = _a[_i];
                var module_1 = akrantiain.findModule(name_1);
                if (module_1 !== undefined) {
                    currentOutput = module_1.convert(currentOutput, akrantiain);
                }
                else {
                    throw new _1.AkrantiainError(9002, -1, "Cannot happen (at Module#convertByModuleChain)");
                }
            }
        }
        return currentOutput;
    };
    Module.prototype.checkUnknownIdentifier = function () {
        for (var _i = 0, _a = this.definitions; _i < _a.length; _i++) {
            var definition = _a[_i];
            var identifier = definition.findUnknownIdentifier(this);
            if (identifier !== undefined) {
                throw new _1.AkrantiainError(1100, -1, "Unresolved identifier: '" + identifier.text + "' in '" + definition + "'");
            }
        }
        for (var _b = 0, _c = this.rules; _b < _c.length; _b++) {
            var rule = _c[_b];
            var identifier = rule.findUnknownIdentifier(this);
            if (identifier !== undefined) {
                throw new _1.AkrantiainError(1101, 335, "Unresolved identifier: '" + identifier.text + "' in '" + rule + "'");
            }
        }
    };
    Module.prototype.checkCircularIdentifier = function () {
        for (var _i = 0, _a = this.definitions; _i < _a.length; _i++) {
            var definition = _a[_i];
            var identifier = definition.findCircularIdentifier([], this);
            if (identifier !== undefined) {
                throw new _1.AkrantiainError(1102, -1, "Circular reference involving identifier: '" + identifier.text + "' in '" + definition + "'");
            }
        }
    };
    Module.prototype.findUnknownModuleName = function (akrantiain) {
        if (this.moduleChain !== undefined) {
            for (var _i = 0, _a = this.moduleChain.names; _i < _a.length; _i++) {
                var name_2 = _a[_i];
                if (akrantiain.findModule(name_2) === undefined) {
                    return name_2;
                }
            }
            return undefined;
        }
        else {
            return undefined;
        }
    };
    Module.prototype.findCircularModuleName = function (names, akrantiain) {
        var _this = this;
        if (this.moduleChain !== undefined) {
            var name_3 = names.find(function (name) { return _this.name === null || name.equals(_this.name); });
            if (name_3 !== undefined) {
                return name_3;
            }
            else {
                var nextNames = (this.name === null) ? names : __spreadArrays(names, [this.name]);
                for (var _i = 0, _a = this.moduleChain.names; _i < _a.length; _i++) {
                    var searchName = _a[_i];
                    var module_2 = akrantiain.findModule(searchName);
                    var name_4 = module_2.findCircularModuleName(nextNames, akrantiain);
                    if (name_4 !== undefined) {
                        return name_4;
                    }
                }
                return undefined;
            }
        }
        else {
            return undefined;
        }
    };
    Module.prototype.findDefinition = function (identifier) {
        for (var _i = 0, _a = this.definitions; _i < _a.length; _i++) {
            var definition = _a[_i];
            if (definition.identifier.equals(identifier)) {
                return definition;
            }
        }
        return undefined;
    };
    Module.prototype.findContent = function (identifier) {
        for (var _i = 0, _a = this.definitions; _i < _a.length; _i++) {
            var definition = _a[_i];
            if (definition.identifier.equals(identifier)) {
                return definition.content;
            }
        }
        return undefined;
    };
    Module.prototype.findPunctuationContent = function () {
        for (var _i = 0, _a = this.definitions; _i < _a.length; _i++) {
            var definition = _a[_i];
            if (definition.identifier.text === "PUNCTUATION") {
                return definition.content;
            }
        }
        return new _1.Disjunction([]);
    };
    Module.prototype.hasDefinition = function (identifier) {
        return this.definitions.findIndex(function (definition) { return definition.identifier.equals(identifier); }) >= 0;
    };
    Module.prototype.hasEnvironment = function (name) {
        return this.environments.findIndex(function (environment) { return environment.name === name; }) >= 0;
    };
    Module.prototype.toString = function (indent) {
        var _a;
        if (indent === void 0) { indent = 0; }
        var string = "";
        string += " ".repeat(indent) + ("% " + ((_a = this.name) !== null && _a !== void 0 ? _a : "<implicit>") + " {\n");
        if (this.definitions.length > 0) {
            string += " ".repeat(indent + 2) + "definitions:\n";
            for (var _i = 0, _b = this.definitions; _i < _b.length; _i++) {
                var definition = _b[_i];
                string += " ".repeat(indent + 4) + (definition + "\n");
            }
        }
        if (this.rules.length > 0) {
            string += " ".repeat(indent + 2) + "rules:\n";
            for (var _c = 0, _d = this.rules; _c < _d.length; _c++) {
                var rule = _d[_c];
                string += " ".repeat(indent + 4) + (rule + "\n");
            }
        }
        if (this.environments.length > 0) {
            string += " ".repeat(indent + 2) + "environments:\n";
            for (var _e = 0, _f = this.environments; _e < _f.length; _e++) {
                var environment = _f[_e];
                string += " ".repeat(indent + 4) + (environment + "\n");
            }
        }
        if (this.moduleChain) {
            string += " ".repeat(indent + 2) + "module chain:\n";
            string += " ".repeat(indent + 4) + (this.moduleChain + "\n");
        }
        string += " ".repeat(indent) + "}\n";
        return string;
    };
    return Module;
}());
exports.Module = Module;
