"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Rule = void 0;
var _1 = require(".");
var Rule = (function () {
    function Rule(ruleLeft, ruleRight) {
        this.selections = ruleLeft.selections;
        this.leftCondition = ruleLeft.leftCondition;
        this.rightCondition = ruleLeft.rightCondition;
        this.phonemes = ruleRight;
        this.checkLength();
        this.checkConcrete();
        this.normalize();
    }
    Rule.prototype.apply = function (stat, module) {
        var _a;
        var appliedGroup = new _1.Stat([]);
        var pointer = 0;
        while (pointer <= stat.elements.length) {
            var result = this.applyOnce(stat, pointer, module);
            if (result !== null) {
                (_a = appliedGroup.elements).push.apply(_a, result.addedElements);
                if (pointer < result.to) {
                    pointer = result.to;
                }
                else {
                    if (pointer < stat.elements.length) {
                        appliedGroup.elements.push(stat.elements[pointer]);
                    }
                    pointer++;
                }
            }
            else {
                if (pointer < stat.elements.length) {
                    appliedGroup.elements.push(stat.elements[pointer]);
                }
                pointer++;
            }
        }
        return appliedGroup;
    };
    Rule.prototype.applyOnce = function (group, from, module) {
        if (this.testLeftCondition(group, from, module)) {
            var result = this.applyOnceSelections(group, from, module);
            if (result !== null) {
                var to = result.to;
                if (this.testRightCondition(group, to, module)) {
                    return result;
                }
                else {
                    return null;
                }
            }
            else {
                return null;
            }
        }
        else {
            return null;
        }
    };
    Rule.prototype.applyOnceSelections = function (group, from, module) {
        var addedElements = [];
        var pointer = from;
        var phonemeIndex = 0;
        for (var _i = 0, _a = this.selections; _i < _a.length; _i++) {
            var selection = _a[_i];
            var to = selection.matchRight(group, pointer, module);
            if (to >= 0) {
                if (selection.isConcrete()) {
                    var phoneme = this.phonemes[phonemeIndex];
                    if (phoneme instanceof _1.Quote || phoneme instanceof _1.Slash) {
                        if (group.isNoneConverted(pointer, to)) {
                            var addedElement = group.merge(pointer, to);
                            addedElement.result = phoneme.text;
                            addedElements.push(addedElement);
                        }
                        else {
                            return null;
                        }
                    }
                    else if (phoneme instanceof _1.Dollar) {
                        for (var i = pointer; i < to; i++) {
                            addedElements.push(group.elements[i]);
                        }
                    }
                    else {
                        throw new Error("This cannot happen");
                    }
                    phonemeIndex++;
                }
                else {
                    for (var i = pointer; i < to; i++) {
                        addedElements.push(group.elements[i]);
                    }
                }
                pointer = to;
            }
            else {
                return null;
            }
        }
        return { addedElements: addedElements, to: pointer };
    };
    Rule.prototype.testLeftCondition = function (group, to, module) {
        if (this.leftCondition !== undefined) {
            var leftStat = group.divide(0, to);
            var rightStat = group.divide(to, group.elements.length);
            return this.leftCondition.matchLeft(leftStat.plus(rightStat), leftStat.elements.length, module) >= 0;
        }
        else {
            return true;
        }
    };
    Rule.prototype.testRightCondition = function (group, from, module) {
        if (this.rightCondition !== undefined) {
            var leftStat = group.divide(0, from);
            var rightStat = group.divide(from, group.elements.length);
            return this.rightCondition.matchRight(leftStat.plus(rightStat), leftStat.elements.length, module) >= 0;
        }
        else {
            return true;
        }
    };
    Rule.prototype.checkLength = function () {
        var concreteSelectionLength = 0;
        var phonemeLength = this.phonemes.length;
        for (var _i = 0, _a = this.selections; _i < _a.length; _i++) {
            var selection = _a[_i];
            if (selection.isConcrete()) {
                concreteSelectionLength++;
            }
        }
        if (concreteSelectionLength !== phonemeLength) {
            throw new _1.AkrantiainError(1104, 333, "Mismatched number of concrete terms in left- and right-hand side: '" + this + "'");
        }
    };
    Rule.prototype.checkConcrete = function () {
        var concrete = false;
        for (var _i = 0, _a = this.phonemes; _i < _a.length; _i++) {
            var phoneme = _a[_i];
            if (!(phoneme instanceof _1.Dollar)) {
                concrete = true;
            }
        }
        if (!concrete) {
            throw new _1.AkrantiainError(1105, 336, "Right-hand side of a sentence consists solely of dollars: '" + this + "'");
        }
    };
    Rule.prototype.normalize = function () {
        var _this = this;
        var leftIndex = 0;
        var rightIndex = this.phonemes.length;
        for (var i = 0; i < this.phonemes.length; i++) {
            var phoneme = this.phonemes[i];
            if (!(phoneme instanceof _1.Dollar)) {
                leftIndex = i;
                break;
            }
        }
        for (var i = this.phonemes.length - 1; i >= 0; i--) {
            var phoneme = this.phonemes[i];
            if (!(phoneme instanceof _1.Dollar)) {
                rightIndex = i + 1;
                break;
            }
        }
        var leftSelectionIndex = 0;
        var rightSelectionIndex = this.selections.length;
        var leftConcreteCount = 0;
        var rightConcreteCount = 0;
        for (var i = 0; i < this.selections.length; i++) {
            var selection = this.selections[i];
            if (selection.isConcrete()) {
                leftConcreteCount++;
            }
            if (leftConcreteCount >= leftIndex + 1) {
                leftSelectionIndex = i;
                break;
            }
        }
        for (var i = this.selections.length - 1; i >= 0; i--) {
            var selection = this.selections[i];
            if (selection.isConcrete()) {
                rightConcreteCount++;
            }
            if (rightConcreteCount >= this.phonemes.length - rightIndex + 1) {
                rightSelectionIndex = i + 1;
                break;
            }
        }
        var nextSelections = this.selections.slice(leftSelectionIndex, rightSelectionIndex);
        var nextLeftCondition = (function () {
            if (_this.leftCondition !== undefined) {
                return new _1.Sequence(__spreadArrays([_this.leftCondition], _this.selections.slice(0, leftSelectionIndex)));
            }
            else {
                if (leftSelectionIndex !== 0) {
                    return new _1.Sequence(_this.selections.slice(0, leftSelectionIndex));
                }
                else {
                    return undefined;
                }
            }
        })();
        var nextRightCondition = (function () {
            if (_this.rightCondition !== undefined) {
                return new _1.Sequence(__spreadArrays(_this.selections.slice(rightSelectionIndex, _this.selections.length), [_this.rightCondition]));
            }
            else {
                if (rightSelectionIndex !== _this.selections.length) {
                    return new _1.Sequence(_this.selections.slice(rightSelectionIndex, _this.selections.length));
                }
                else {
                    return undefined;
                }
            }
        })();
        var nextPhonemes = this.phonemes.slice(leftIndex, rightIndex);
        this.selections = nextSelections;
        this.leftCondition = nextLeftCondition;
        this.rightCondition = nextRightCondition;
        this.phonemes = nextPhonemes;
    };
    Rule.prototype.findUnknownIdentifier = function (module) {
        for (var _i = 0, _a = this.selections; _i < _a.length; _i++) {
            var selection = _a[_i];
            var identifier = selection.findUnknownIdentifier(module);
            if (identifier !== undefined) {
                return identifier;
            }
        }
        if (this.leftCondition !== undefined) {
            var identifier = this.leftCondition.findUnknownIdentifier(module);
            if (identifier !== undefined) {
                return identifier;
            }
        }
        if (this.rightCondition !== undefined) {
            var identifier = this.rightCondition.findUnknownIdentifier(module);
            if (identifier !== undefined) {
                return identifier;
            }
        }
        return undefined;
    };
    Rule.prototype.toString = function () {
        var string = "";
        if (this.leftCondition) {
            string += "[" + this.leftCondition.toString() + "] ";
        }
        string += this.selections.join(" ");
        if (this.rightCondition) {
            string += " [" + this.rightCondition.toString() + "]";
        }
        string += " -> ";
        string += this.phonemes.join(" ");
        string += ";";
        return string;
    };
    return Rule;
}());
exports.Rule = Rule;
