"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sequence = void 0;
var Sequence = (function () {
    function Sequence(matchables) {
        this.matchables = matchables;
    }
    Sequence.prototype.matchRight = function (stat, from, module) {
        if (this.matchables.length > 0) {
            var pointer = from;
            for (var _i = 0, _a = this.matchables; _i < _a.length; _i++) {
                var matchable = _a[_i];
                var to = matchable.matchRight(stat, pointer, module);
                if (to >= 0) {
                    pointer = to;
                }
                else {
                    return -1;
                }
            }
            return pointer;
        }
        else {
            return from;
        }
    };
    Sequence.prototype.matchLeft = function (stat, to, module) {
        if (this.matchables.length > 0) {
            var pointer = to;
            for (var _i = 0, _a = this.matchables.slice().reverse(); _i < _a.length; _i++) {
                var matchable = _a[_i];
                var from = matchable.matchLeft(stat, pointer, module);
                if (from >= 0) {
                    pointer = from;
                }
                else {
                    return -1;
                }
            }
            return pointer;
        }
        else {
            return to;
        }
    };
    Sequence.prototype.isConcrete = function () {
        return this.matchables.length >= 2 || (this.matchables.length >= 1 && this.matchables[0].isConcrete());
    };
    Sequence.prototype.findUnknownIdentifier = function (module) {
        for (var _i = 0, _a = this.matchables; _i < _a.length; _i++) {
            var matchable = _a[_i];
            var identifier = matchable.findUnknownIdentifier(module);
            if (identifier !== undefined) {
                return identifier;
            }
        }
        return undefined;
    };
    Sequence.prototype.findCircularIdentifier = function (identifiers, module) {
        for (var _i = 0, _a = this.matchables; _i < _a.length; _i++) {
            var matchable = _a[_i];
            var identifier = matchable.findCircularIdentifier(identifiers, module);
            if (identifier !== undefined) {
                return identifier;
            }
        }
        return undefined;
    };
    Sequence.prototype.toString = function () {
        var string = "";
        string += this.matchables.join(" ");
        return string;
    };
    return Sequence;
}());
exports.Sequence = Sequence;
