"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Stat = void 0;
var _1 = require(".");
var Stat = (function () {
    function Stat(elements) {
        this.elements = elements;
    }
    Stat.create = function (input, module) {
        if (!module.hasEnvironment("CASE_SENSITIVE") && !module.hasEnvironment("PRESERVE_CASE")) {
            input = input.toLowerCase();
        }
        if (module.hasEnvironment("USE_NFD")) {
            input = input.normalize("NFD");
        }
        var elements = [];
        for (var i = 0; i < input.length; i++) {
            var element = new _1.StatElement(input[i], null, i + 1);
            elements.push(element);
        }
        var stat = new Stat(elements);
        return stat;
    };
    Stat.prototype.merge = function (from, to) {
        var part = "";
        for (var i = from; i < to; i++) {
            part += this.elements[i].part;
        }
        var columnNumber = (from < this.elements.length) ? this.elements[from].columnNumber : this.elements[this.elements.length - 1].columnNumber;
        var element = new _1.StatElement(part, null, columnNumber);
        return element;
    };
    Stat.prototype.divide = function (from, to) {
        var elements = [];
        for (var i = from; i < to; i++) {
            elements.push.apply(elements, this.elements[i].divide());
        }
        var stat = new Stat(elements);
        return stat;
    };
    Stat.prototype.plus = function (that) {
        var elements = [];
        elements.push.apply(elements, this.elements);
        elements.push.apply(elements, that.elements);
        var stat = new Stat(elements);
        return stat;
    };
    Stat.prototype.createOutput = function (module) {
        var output = "";
        for (var _i = 0, _a = this.elements; _i < _a.length; _i++) {
            var element = _a[_i];
            if (element.result !== null) {
                output += element.result;
            }
            else {
                if (element.isValid(module)) {
                    output += " ";
                }
                else {
                    output += element.part;
                }
            }
        }
        if (module.hasEnvironment("USE_NFD")) {
            output = output.normalize("NFD");
        }
        return output;
    };
    Stat.prototype.getInvalidElements = function (module) {
        var invalidElements = [];
        for (var _i = 0, _a = this.elements; _i < _a.length; _i++) {
            var element = _a[_i];
            if (!element.isValid(module)) {
                invalidElements.push(element);
            }
        }
        return invalidElements;
    };
    Stat.prototype.isAllConverted = function (from, to) {
        var result = true;
        for (var i = from; i < to; i++) {
            if (!this.elements[i].isConverted()) {
                result = false;
                break;
            }
        }
        return result;
    };
    Stat.prototype.isNoneConverted = function (from, to) {
        var result = true;
        for (var i = from; i < to; i++) {
            if (this.elements[i].isConverted()) {
                result = false;
                break;
            }
        }
        return result;
    };
    return Stat;
}());
exports.Stat = Stat;
