"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Parsers = void 0;
var parsimmon_1 = __importDefault(require("parsimmon"));
var parsimmon_2 = require("parsimmon");
var class_1 = require("../class");
var util_1 = require("./util");
var Parsers = (function () {
    function Parsers() {
    }
    Parsers.parened = function (parser) {
        var leftParser = parsimmon_2.seq(parsimmon_1.default.string("("), Parsers.blank);
        var rightParser = parsimmon_2.seq(Parsers.blank, parsimmon_1.default.string(")"));
        var wrappedParser = parsimmon_2.seq(leftParser, parser, rightParser).map(function (result) { return result[1]; });
        return wrappedParser;
    };
    Parsers.akrantiain = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(Parsers.blankOrBreak, parsimmon_2.alt(Parsers.implicitModule, Parsers.explicitModule).sepBy(Parsers.blankOrBreak), Parsers.blankOrBreak, parsimmon_1.default.eof).map(function (_a) {
            var modules = _a[1];
            return new class_1.Akrantiain(modules);
        });
        return parser;
    });
    Parsers.implicitModule = parsimmon_2.lazy(function () {
        var parser = Parsers.sentences.map(function (sentences) { return new class_1.Module(null, sentences); });
        return parser;
    });
    Parsers.explicitModule = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(parsimmon_2.seq(parsimmon_1.default.string("%"), Parsers.blank), Parsers.moduleName, parsimmon_2.seq(Parsers.blankOrBreak, parsimmon_1.default.string("{"), Parsers.blankOrBreak), Parsers.sentences, parsimmon_2.seq(Parsers.blankOrBreak, parsimmon_1.default.string("}"))).map(function (_a) {
            var name = _a[1], sentences = _a[3];
            return new class_1.Module(name, sentences);
        });
        return parser;
    });
    Parsers.sentences = parsimmon_2.lazy(function () {
        var parser = Parsers.sentence.atLeast(1).map(function (sentences) {
            var filteredSentences = sentences.filter(function (sentence) { return sentence !== null; });
            return filteredSentences;
        });
        return parser;
    });
    Parsers.sentence = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.alt(Parsers.definition, Parsers.rule, Parsers.environment, Parsers.moduleChain, Parsers.comment);
        return parser;
    });
    Parsers.definition = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(Parsers.identifier, parsimmon_1.default.string("=").trim(Parsers.blank), Parsers.disjunction, parsimmon_2.seq(Parsers.blank, Parsers.semicolon)).map(function (_a) {
            var identifier = _a[0], content = _a[2];
            return new class_1.Definition(identifier, content);
        });
        return parser;
    });
    Parsers.rule = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(Parsers.ruleLeft, parsimmon_1.default.string("->").trim(Parsers.blank), Parsers.ruleRight, parsimmon_2.seq(Parsers.blank, Parsers.semicolon)).map(function (_a) {
            var ruleLeft = _a[0], ruleRight = _a[2];
            return new class_1.Rule(ruleLeft, ruleRight);
        });
        return parser;
    });
    Parsers.ruleLeft = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(Parsers.condition.times(0, 1).map(function (result) { return result[0]; }), Parsers.blank, Parsers.selection.sepBy1(Parsers.blank), Parsers.blank, Parsers.condition.times(0, 1).map(function (result) { return result[0]; })).map(function (_a) {
            var leftCondition = _a[0], selections = _a[2], rightCondition = _a[4];
            return ({ selections: selections, leftCondition: leftCondition, rightCondition: rightCondition });
        });
        return parser;
    });
    Parsers.ruleRight = parsimmon_2.lazy(function () {
        var elementParser = parsimmon_2.alt(Parsers.quote, Parsers.slash, Parsers.dollar);
        var parser = elementParser.sepBy1(Parsers.blank);
        return parser;
    });
    Parsers.disjunction = parsimmon_2.lazy(function () {
        var parser = Parsers.sequence.sepBy1(parsimmon_1.default.string("|").trim(Parsers.blank)).map(function (sequences) { return new class_1.Disjunction(sequences, false); });
        return parser;
    });
    Parsers.sequence = parsimmon_2.lazy(function () {
        var parser = Parsers.selection.sepBy1(Parsers.blank).map(function (selections) { return new class_1.Sequence(selections); });
        return parser;
    });
    Parsers.condition = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(parsimmon_1.default.string("!"), Parsers.blank, Parsers.selection).map(function (_a) {
            var selection = _a[2];
            return new class_1.Disjunction([selection], true);
        });
        return parser;
    });
    Parsers.selection = parsimmon_2.lazy(function () {
        var disjunctionParser = Parsers.disjunction.thru(Parsers.parened);
        var parser = parsimmon_2.alt(Parsers.quote, Parsers.circumflex, Parsers.identifier, disjunctionParser);
        return parser;
    });
    Parsers.environment = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(parsimmon_2.seq(parsimmon_1.default.string("@"), Parsers.blank), Parsers.identifierString, parsimmon_2.seq(Parsers.blank, Parsers.semicolon)).map(function (_a) {
            var rawName = _a[1];
            return new class_1.Environment(rawName);
        });
        return parser;
    });
    Parsers.moduleChain = parsimmon_2.lazy(function () {
        var chainElementParser = parsimmon_2.alt(Parsers.moduleChainElement, Parsers.moduleChainElement.thru(Parsers.parened));
        var chainParser = chainElementParser.sepBy1(parsimmon_1.default.string(">>").trim(Parsers.blank)).map(function (chainElements) {
            var modules = [];
            for (var _i = 0, chainElements_1 = chainElements; _i < chainElements_1.length; _i++) {
                var chainElement = chainElements_1[_i];
                modules.push.apply(modules, chainElement);
            }
            var chain = new class_1.ModuleChain(modules);
            return chain;
        });
        var parser = parsimmon_2.seq(parsimmon_2.seq(parsimmon_1.default.string("%%"), Parsers.blank), chainParser, parsimmon_2.seq(Parsers.blank, Parsers.semicolon)).map(function (_a) {
            var chain = _a[1];
            return chain;
        });
        return parser;
    });
    Parsers.moduleChainElement = parsimmon_2.lazy(function () {
        var parser = Parsers.identifierString.sepBy1(parsimmon_1.default.string("=>").trim(Parsers.blank)).map(function (texts) {
            if (texts.length === 1) {
                var name_1 = new class_1.ModuleName(texts[0]);
                return [name_1];
            }
            else {
                var names = [];
                for (var i = 0; i < texts.length - 1; i++) {
                    var name_2 = new class_1.ModuleName(texts[i], texts[i + 1]);
                    names.push(name_2);
                }
                return names;
            }
        });
        return parser;
    });
    Parsers.moduleName = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.alt(Parsers.moduleChainName.thru(util_1.attempt), Parsers.moduleSimpleName);
        return parser;
    });
    Parsers.moduleSimpleName = parsimmon_2.lazy(function () {
        return Parsers.identifierString.map(function (text) { return new class_1.ModuleName(text); });
    });
    Parsers.moduleChainName = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(Parsers.identifierString, parsimmon_1.default.string("=>").trim(Parsers.blank), Parsers.identifierString).map(function (_a) {
            var text = _a[0], extraText = _a[2];
            return new class_1.ModuleName(text, extraText);
        });
        return parser;
    });
    Parsers.quote = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(parsimmon_1.default.string("\""), parsimmon_2.alt(Parsers.quoteEscape, Parsers.quoteContent).many().tie(), parsimmon_1.default.string("\"")).map(function (_a) {
            var string = _a[1];
            return new class_1.Quote(string);
        });
        return parser;
    });
    Parsers.quoteEscape = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(parsimmon_1.default.string("\\"), parsimmon_2.alt(parsimmon_1.default.regexp(/u[A-Fa-f0-9]{4}/), parsimmon_1.default.oneOf("\\\""))).map(function (_a) {
            var escape = _a[1];
            if (escape.startsWith("u")) {
                var code = parseInt(escape.substr(1, 4), 16);
                var char = String.fromCharCode(code);
                return char;
            }
            else {
                return escape;
            }
        });
        return parser;
    });
    Parsers.quoteContent = parsimmon_2.lazy(function () {
        var parser = parsimmon_1.default.noneOf("\\\"");
        return parser;
    });
    Parsers.slash = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(parsimmon_1.default.string("/"), parsimmon_2.alt(Parsers.slashEscape, Parsers.slashContent).many().tie(), parsimmon_1.default.string("/")).map(function (_a) {
            var string = _a[1];
            return new class_1.Slash(string);
        });
        return parser;
    });
    Parsers.slashEscape = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(parsimmon_1.default.string("\\"), parsimmon_2.alt(parsimmon_1.default.regexp(/u[A-Fa-f0-9]{4}/), parsimmon_1.default.oneOf("\\/"))).map(function (_a) {
            var escape = _a[1];
            if (escape.startsWith("u")) {
                var code = parseInt(escape.substr(1, 4), 16);
                var char = String.fromCharCode(code);
                return char;
            }
            else {
                return escape;
            }
        });
        return parser;
    });
    Parsers.slashContent = parsimmon_2.lazy(function () {
        var parser = parsimmon_1.default.noneOf("\\/");
        return parser;
    });
    Parsers.circumflex = parsimmon_2.lazy(function () {
        var parser = parsimmon_1.default.string("^").result(new class_1.Circumflex());
        return parser;
    });
    Parsers.dollar = parsimmon_2.lazy(function () {
        var parser = parsimmon_1.default.string("$").result(new class_1.Dollar());
        return parser;
    });
    Parsers.comment = parsimmon_2.lazy(function () {
        var parser = parsimmon_2.seq(parsimmon_1.default.string("#"), parsimmon_1.default.noneOf("\n").many(), Parsers.blankOrBreak).result(null);
        return parser;
    });
    Parsers.semicolon = parsimmon_2.lazy(function () {
        var semicolonParser = parsimmon_2.seq(parsimmon_1.default.string(";"), Parsers.blankOrBreak);
        var breakParser = parsimmon_2.seq(Parsers.break, Parsers.blankOrBreak);
        var otherParser = parsimmon_1.default.lookahead(parsimmon_2.alt(parsimmon_1.default.string("#"), parsimmon_1.default.string("}"), parsimmon_1.default.eof));
        var parser = parsimmon_2.alt(semicolonParser, breakParser, otherParser).result(null);
        return parser;
    });
    Parsers.identifier = parsimmon_2.lazy(function () {
        var parser = Parsers.identifierString.map(function (string) { return new class_1.Identifier(string); });
        return parser;
    });
    Parsers.identifierString = parsimmon_2.lazy(function () {
        var parser = parsimmon_1.default.regexp(/[a-zA-Z][a-zA-Z0-9_]*/);
        return parser;
    });
    Parsers.blankOrBreak = parsimmon_2.lazy(function () {
        var parser = parsimmon_1.default.regexp(/\s*/).result(null);
        return parser;
    });
    Parsers.blank = parsimmon_2.lazy(function () {
        var parser = parsimmon_1.default.regexp(/[^\S\n]*/).result(null);
        return parser;
    });
    Parsers.break = parsimmon_2.lazy(function () {
        var parser = parsimmon_1.default.string("\n").result(null);
        return parser;
    });
    return Parsers;
}());
exports.Parsers = Parsers;
